require 'fluent/plugin/filter'
require 'json'
require 'socket'

module Fluent
  module Plugin
    # Filter plugin that extracts health metrics from OnTap xml
    class PrometheusAlertFilter < Filter
      Fluent::Plugin.register_filter('prometheus_alert', self)

      config_param :host, :string, :default => nil
      config_param :port, :integer, :default => 8125
      config_param :namespace, :string, :default => 'Demo'

      def configure(conf)
        super
      end

      def filter(_tag, _time, record)
        records = []

        record['alerts'].each do |alert|
          dims = {}

          # Dimensions for hotpath - metrics and warmpath-logs, These can not be null/empty
          dims['AlertName'] = alert.dig('labels', 'alertname')
          dims['Severity'] = alert.dig('labels', 'severity')
          dims['Status'] = alert.dig('status')

          if record.key?('datacenter')
            dims['DataCenter'] = record['datacenter']
          else
            dims['DataCenter'] = 'attn: Datacenter Variable not set'
            $stdout.write 'Warning: Datacenter environment variable not set'
          end

          statsd_message = {}
          statsd_message['Namespace'] = @namespace
          statsd_message['Metric'] = 'PrometheusAlert'
          statsd_message['Dims'] = dims
          statsd_json = statsd_message.to_json
          statsd_payload = "#{statsd_json}:1|g"

          # send to hotpath - metrics
          s = UDPSocket.new
          s.send(statsd_payload, 0, @host, @port)
          s.close

          # Dimensions for warmpath-logs, These can be empty / null.
          dims['Component'] = alert.dig('labels', 'component')
          dims['Job'] = alert.dig('labels', 'job')
          dims['Pod'] = alert.dig('labels', 'pod')
          dims['Container'] = alert.dig('labels', 'container')
          dims['Description'] = alert.dig('annotations', 'description')

          statsd_message['Dims'] = dims

          # return to warmpath
          records.push(statsd_message)
        end
        records
      end
    end
  end
end